
import argparse
import contextlib
import dataclasses
import functools
import os
import tomllib
import typing
from importlib import resources

from petrus._core.calcs.Prog import Prog

__all__ = ["main", "run"]

def _desc():
    return "Update a python project. \
The default values of the arguments are taken \
from the default table of the config.toml file \
inside of the petrus package."

def _inputs():
    pairs = list(dict(Prog.INPUTS).items())
    #pairs = list(_input_format(x) for x in pairs)
    pairs.sort(key=_inputs_sortkey)
    ans = dict(pairs)
    return ans

def _inputs_sortkey(pair):
    if pair[0] == "path":
        raise KeyError
    if "-" in pair[0]:
        raise KeyError
    return pair[0]

def _run_deco(old, /):
    doc = _desc()
    doc += "\n"
    for k, v in _inputs().items():
        old.__annotations__[k] = typing.Optional[str]
        setattr(old, k, None)
        doc += f"\n{k}: {v}"
    old = dataclasses.dataclass(old)
    @functools.wraps(old)
    def new(*args, **kwargs):
        old(*args, **kwargs)
    new.__doc__ = doc
    return new

def main(args=None):
    parser = argparse.ArgumentParser(
        fromfile_prefix_chars="@",
        description=_desc())
    parser.add_argument("path", nargs="?")
    for k, v in _inputs().items():
        opt = "--" + k.replace("_", "-")
        parser.add_argument(opt, help=v)
    ns = parser.parse_args(args)
    kwargs = vars(ns)
    _prog(**kwargs)

@_run_deco
class run:
    path:typing.Optional[str]=None
    def __post_init__(self):
        kwargs = dataclasses.asdict(self)
        _prog(**kwargs)

def _prog(path, **kwargs):
    try:
        cfg = resources.read_text("petrus", "config.toml")
    except:
        cfg = ""
    cfg = tomllib.loads(cfg)
    default = cfg.get("default", {})
    for k in kwargs.keys():
        if kwargs[k] is None:
            kwargs[k] = str(default.get(k, ""))
    try:
        root = cfg["general"]["root"]
    except KeyError:
        root = None
    paths = [os.getcwd(), root, path]
    paths = filter(lambda x:x is not None, paths)
    paths = [_normpath(x) for x in paths]
    wd = os.path.join(*paths)
    if not os.path.isdir(wd):
        os.mkdir(wd)
    with contextlib.chdir(wd):
        Prog(kwargs)

def _normpath(path):
    path = os.path.expanduser(path)
    path = os.path.expandvars(path)
    path = os.path.normpath(path)
    return path



import importlib.resources

from petrus._core.calcs.Calc import Calc


class Draft(Calc):
    def _calc(self, name):
        return importlib.resources.read_text(
            "petrus.drafts", 
            "%s.txt" % name)




import os
import subprocess

from petrus._core.calcs.Calc import Calc


class Git(Calc):
    def __call__(self, *args, force=False):
        for a in args:
            if type(a) is not str:
                raise TypeError(a)
        if not force and not self.is_repo():
            return
        try:
            subprocess.run(["git", "rev-parse"], check=True)
        except subprocess.CalledProcessError:
            return
        args = ["git"] + list(args)
        return subprocess.run(args, check=False)
    def _calc_author(self):
        a = self.prog.kwargs["author"]
        e = self.prog.kwargs["email"]
        if {a, e} == {None}:
            return None
        if a is None:
            return e
        if e is None:
            return a
        return f"{a} <{e}>"
    def init(self):
        if self.is_repo():
            return 
        self("init", os.getcwd(), force=True)
        self.commit("Initial Commit")
    def commit_version(self):
        m = "Version %s" % self.prog.project.version
        self.commit(m)
    def commit(self, message):
        if message is None:
            message = "a"
        else:
            message = str(message)
        try:
            self('add', '-A').check_returncode()
        except:
            return
        args = ['commit', '--allow-empty', '-m', message]
        if self.author is not None:
            args += ['--author', self.author]
        self(*args)
    def push(self):
        ...
    def is_repo(self):
        called = self("rev-parse", force=True)
        return not called.returncode
    def move(self, a, b, /):
        try:
            self("mv", a, b).check_returncode()
        except:
            pass
        else:
            return
        os.rename(a, b)
       





import inspect
import os
import string
import sys
import tomllib

import tomli_w

from petrus._core import utils
from petrus._core.calcs.Calc import Calc
from petrus._core.Version import Version


class Project(Calc):
    def _calc_authors(self):
        ans = self.get("authors", default=[])
        if type(ans) is not list:
            return ans
        ans = list(ans)
        author = dict()
        a = dict()
        a["name"] = self.prog.kwargs["author"]
        a["email"] = self.prog.kwargs["email"]
        for k, v in a.items():
            if v:
                author[k] = v
        author = utils.easy_dict(author)
        used = False
        for i in range(len(ans)):
            try:
                ans[i] = dict(ans[i])
            except:
                continue
            fit = utils.dict_match(ans[i], author)
            if fit and not used:
                ans[i].update(author)
            ans[i] = utils.easy_dict(ans[i])
            used |= fit
        if not used:
            ans.insert(0, author)
        return ans
    def _calc_classifiers(self):
        ans = self.get("classifiers")
        if ans is not None:
            return ans
        ans = [
            "Programming Language :: Python",
            "Programming Language :: Python :: 3"]
        if not utils.isfile(self.prog.file.license):
            ans += ["License :: OSI Approved :: MIT License"]
        ans = utils.easy_list(ans)
        return ans
    def _calc_dependencies(self):
        ans = self.get("dependencies", default=[])
        if type(ans) is not list:
            return ans
        ans = [utils.fix_dependency(x) for x in ans]
        ans = utils.easy_list(ans)
        return ans
    def _calc_description(self):
        return (
            self.prog.kwargs["description"] 
            or self.name)
    def _calc_keywords(self):
        return self.get("keywords", default=[])
    def _calc_license(self):
        ans = self.get("license")
        if ans is None:
            ans = dict()
        if type(ans) is not dict:
            return ans
        if "file" not in ans.keys():
            ans["file"] = self.prog.file.license
        return ans
    def _calc_name(self):
        basename = os.path.basename(os.getcwd())
        return self.get("name") or basename
    def _calc_readme(self):
        return self.prog.file.readme
    def _calc_requires_python(self):
        return (
            self.prog.kwargs["requires_python"] 
            or ">={0}.{1}.{2}".format(*sys.version_info))
    def _calc_urls(self):
        ans = self.get("urls")
        if ans is None:
            ans = dict()
        if type(ans) is not dict:
            return ans
        if self.prog.github:
            ans.setdefault("Source", self.prog.github)
        p = f"https://pypi.org/project/{self.name}/#files"
        ans.setdefault("Download", p)
        ans = utils.easy_dict(ans)
        return ans
    def _calc_version(self):
        a = self.prog.kwargs["project_version"]
        b = self.get("version", default="0.0.0")
        c = str(Version.parse(b).apply(a))
        return c
    def get(self, *args, default=None):
        args = ("project",) + tuple(args)
        ans = self.prog.pp[args]
        if ans is None:
            return default
        return ans
    def to_dict(self) -> None:
        ans = self.get()
        prefix = "_calc_"
        for n, m in inspect.getmembers(self):
            if not n.startswith(prefix):
                continue
            k = n[len(prefix):]
            v = getattr(self, k)
            ans[k.replace("_", "-")] = v
        ans = utils.easy_dict(ans)
        return ans


 